--- old/usr/src/cmd/sgs/tools/common/leb128.c	2015-03-23 21:41:49.901926642 -0400
+++ new/usr/src/cmd/sgs/tools/common/leb128.c	2015-03-23 21:41:49.984834993 -0400
@@ -99,8 +99,8 @@
  *
  */
 
-uint64_t
-uleb_extract(unsigned char *data, uint64_t *dotp)
+dwarf_error_t
+uleb_extract(unsigned char *data, uint64_t *dotp, size_t len, uint64_t *ret)
 {
 	uint64_t	dot = *dotp;
 	uint64_t	res = 0;
@@ -111,6 +111,9 @@
 	data += dot;
 
 	while (more) {
+		if (dot > len)
+			return (DW_OVERFLOW);
+
 		/*
 		 * Pull off lower 7 bits
 		 */
@@ -134,11 +137,12 @@
 		more = ((*data++) & 0x80) >> 7;
 	}
 	*dotp = dot;
-	return (res);
+	*ret = res;
+	return (DW_SUCCESS);
 }
 
-int64_t
-sleb_extract(unsigned char *data, uint64_t *dotp)
+dwarf_error_t
+sleb_extract(unsigned char *data, uint64_t *dotp, size_t len, int64_t *ret)
 {
 	uint64_t	dot = *dotp;
 	int64_t		res = 0;
@@ -149,6 +153,9 @@
 	data += dot;
 
 	while (more) {
+		if (dot > len)
+			return (DW_OVERFLOW);
+
 		/*
 		 * Pull off lower 7 bits
 		 */
@@ -177,8 +184,8 @@
 	 * Make sure value is properly sign extended.
 	 */
 	res = (res << (64 - shift)) >> (64 - shift);
-
-	return (res);
+	*ret = res;
+	return (DW_SUCCESS);
 }
 
 /*
@@ -196,10 +203,11 @@
  *	dbase - The base address to which DW_EH_PE_datarel is relative
  *		(if frame_hdr is false)
  */
-uint64_t
-dwarf_ehe_extract(unsigned char *data, uint64_t *dotp, uint_t ehe_flags,
-    unsigned char *eident, boolean_t frame_hdr, uint64_t sh_base,
-    uint64_t sh_offset, uint64_t dbase)
+dwarf_error_t
+dwarf_ehe_extract(unsigned char *data, size_t len, uint64_t *dotp,
+    uint64_t *ret, uint_t ehe_flags, unsigned char *eident,
+    boolean_t frame_hdr, uint64_t sh_base, uint64_t sh_offset,
+    uint64_t dbase)
 {
 	uint64_t    dot = *dotp;
 	uint_t	    lsb;
@@ -219,7 +227,8 @@
 
 	switch (ehe_flags & 0x0f) {
 	case DW_EH_PE_omit:
-		return (0);
+		*ret = 0;
+		return (DW_SUCCESS);
 	case DW_EH_PE_absptr:
 		fsize = wordsize;
 		break;
@@ -236,11 +245,12 @@
 		fsize = 2;
 		break;
 	case DW_EH_PE_uleb128:
-		return (uleb_extract(data, dotp));
+		return (uleb_extract(data, dotp, len, ret));
 	case DW_EH_PE_sleb128:
-		return ((uint64_t)sleb_extract(data, dotp));
+		return (sleb_extract(data, dotp, len, (int64_t *)ret));
 	default:
-		return (0);
+		*ret = 0;
+		return (DW_BAD_ENCODING);
 	}
 
 	if (lsb) {
@@ -253,6 +263,9 @@
 		for (cnt = 0; cnt < fsize;
 		    cnt++, dot++) {
 			uint64_t val;
+
+			if (dot > len)
+				return (DW_OVERFLOW);
 			val = data[dot];
 			result |= val << (cnt * 8);
 		}
@@ -264,7 +277,10 @@
 		result = 0;
 		for (cnt = 0; cnt < fsize;
 		    cnt++, dot++) {
-			uint64_t	val;
+			uint64_t val;
+
+			if (dot > len)
+				return (DW_OVERFLOW);
 			val = data[dot];
 			result |= val << ((fsize - cnt - 1) * 8);
 		}
@@ -307,5 +323,6 @@
 	    ((sizeof (uint64_t) - fsize) * 8);
 
 	*dotp = dot;
-	return (result);
+	*ret = result;
+	return (DW_SUCCESS);
 }
